# Main file of CX-Deploy
# -*- coding: utf-8 -*-

import sys
import os
from os import path
from PyQt5.QtWidgets import QMainWindow, QApplication, QPushButton, QFileDialog, QLineEdit, QWidget, QMessageBox
from PyQt5 import uic


import cfg
from general_functions import *
import cli_functions
import file_functions
import html_target
import glfw_target

# This is necessary for pyinstaller, to get access to the file even in --onefile exe bundle.
cfg.bundle_dir = getattr(sys, '_MEIPASS', path.abspath(path.dirname(__file__)))
ui_file_path = path.join(cfg.bundle_dir, 'CXDeployUI.ui')
Ui_MainWindow, QtBaseClass = uic.loadUiType(ui_file_path)


class MyApp(QMainWindow):
    def __init__(self):
        super(MyApp, self).__init__()
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)


        # Connect Gui Methods to widget signals.
        # General Tab
        self.ui.toolButton_ProjectMainFile.clicked.connect(self.toolButton_ProjectMainFile_Clicked)
        #self.ui.lineEdit_ProjectMainFile.textChanged.connect(self.lineEdit_ProjectMainFile_textChanged)
        self.ui.lineEdit_ProjectMainFile.editingFinished.connect(self.lineEdit_ProjectMainFile_editingFinished)
        self.ui.lineEdit_ProjectName.textChanged.connect(self.lineEdit_ProjectName_textChanged)
        self.ui.lineEdit_ProjectName.textEdited.connect(self.lineEdit_ProjectName_textEdited)

        # Html Tab
        self.ui.lineEdit_HtmlAppName.textChanged.connect(self.lineEdit_HtmlAppName_textChanged)
        self.ui.lineEdit_HtmlAppName.textEdited.connect(self.lineEdit_HtmlAppName_textEdited)
        self.ui.toolButton_HtmlFaviconFile.clicked.connect(self.toolButton_HtmlFaviconFile_clicked)
        self.ui.pushButton_HtmlMakeProduct.clicked.connect(self.pushButton_HtmlMakeProduct_clicked)

        # Glfw Tab
        #self.ui.comboBox_GlfwRelDir.currentTextChanged.connect(self.comboBox_GlfwRelDir_currentTextChanged)
        self.ui.lineEdit_GlfwAppName.textChanged.connect(self.lineEdit_GlfwAppName_textChanged)
        self.ui.lineEdit_GlfwAppName.textEdited.connect(self.lineEdit_GlfwAppName_textEdited)
        self.ui.lineEdit_GlfwAppVersion.textChanged.connect(self.lineEdit_GlfwAppVersion_textChanged)
        self.ui.lineEdit_GlfwAppVersion.textEdited.connect(self.lineEdit_GlfwAppVersion_textEdited)
        self.ui.lineEdit_GlfwAppExeFileName.textEdited.connect(self.lineEdit_GlfwAppExeFileName_textEdited)
        self.ui.lineEdit_GlfwAppId.textEdited.connect(self.lineEdit_GlfwAppId_textEdited)
        self.ui.pushButton_GlfwMakeProduct.clicked.connect(self.pushButton_GlfwMakeProduct_clicked)


        # Trigger the ProjDirChange at start to autofill all lineEdits.
        if cfg.args.mainfile and os.path.isfile(cfg.args.mainfile):
            self.ui.lineEdit_ProjectMainFile.setText(cfg.args.mainfile)
            self.lineEdit_ProjectMainFile_editingFinished()

        # Insist, that a CX Project Dir is there.
        if self.ui.lineEdit_ProjectMainFile.text() == "" or not os.path.isfile(self.ui.lineEdit_ProjectMainFile.text()):
            print("No valid ProjectFile!")
            self.toolButton_ProjectMainFile_Clicked()

        # Trigger the update of the AppVersion to the current date yyyy-mm-dd.
        if self.ui.checkBox_GlfwAppVersion.isChecked():
            self.ui.lineEdit_GlfwAppVersion.setText(getDateString())


    def closeEvent(self, event):
        print("close")
        #reply = QMessageBox.question(self, 'Message',
        #                             "Save changes?", QMessageBox.Yes |
        #                           QMessageBox.No, QMessageBox.No)
        reply = QMessageBox.Yes
        if reply == QMessageBox.Yes:
            file_functions.save_widgets_to_file(self)
            event.accept()
        else:
            event.ignore()

    # Check, if CXDeployDir is there and create it, if not.
    def check_CXDeployDir(self):
        old_cwd=os.getcwd()
        if not os.path.isdir(cfg.CXDEPLOY_DIRNAME):
            os.mkdir(cfg.CXDEPLOY_DIRNAME)
            os.chdir(cfg.CXDEPLOY_DIRNAME)
            os.mkdir(cfg.CXDEPLOY_PREFSDIRNAME)
            os.mkdir(cfg.CXDEPLOY_PRODUCTSDIRNAME)
            os.chdir(old_cwd)

    # Methods for GUI Behaviour.
    # General Tab
    def lineEdit_ProjectMainFile_editingFinished(self):
        # Set CXDeployDirs
        main_file = self.ui.lineEdit_ProjectMainFile.text()
        if not self.check_project_mainfile(main_file):
            print("no valid main file -> returning to last one")
            self.ui.lineEdit_ProjectMainFile.setText(os.path.join(cfg.project_dir, cfg.project_filename))
            return
        cfg.project_dir = os.path.split(main_file)[0]
        cfg.project_filename = os.path.split(main_file)[1]
        print("ProjectFilename: " + cfg.project_filename)
        project_name = os.path.splitext(cfg.project_filename)[0]

        print("ProjectDir: " + cfg.project_dir)
        os.chdir(cfg.project_dir)
        self.check_CXDeployDir()
        cfg.CXDeployDir = os.path.abspath(cfg.CXDEPLOY_DIRNAME)
        os.chdir(cfg.CXDeployDir)
        cfg.prefs_dir = os.path.abspath(cfg.CXDEPLOY_PREFSDIRNAME)
        cfg.products_dir = os.path.abspath(cfg.CXDEPLOY_PRODUCTSDIRNAME)
        # print("PrefsDir: "+ cfg.PrefsDir)
        # print("ProductsDir: "+ cfg.ProductsDir)

        # Auto fill relative build directory by searching for .build in project dir.
        self.ui.comboBox_RelBuildDir.clear()
        for dir_name in os.listdir(cfg.project_dir):
            if dir_name.find(project_name + ".build") >= 0:
                self.ui.comboBox_RelBuildDir.addItem(dir_name)

        # Load json-file with widget-settings except projectmainfile.
        file_functions.load_file_to_widgets(self)

        # Auto fill Project name, starting auto fill chain.
        if self.ui.checkBox_ProjectName.isChecked():
            self.ui.lineEdit_ProjectName.setText(project_name)



    def check_project_mainfile(self, main_file):
        print("dir to check: " + main_file)
        if main_file and os.path.isfile(main_file) and os.path.splitext(main_file)[1]==".cxs":
            return True
        else:
            return False

    def toolButton_ProjectMainFile_Clicked(self):
        while True:
            current_main_file = self.ui.lineEdit_ProjectMainFile.text()
            current_dir = os.path.split(current_main_file)[0]
            print("currentdir: " + current_dir)
            new_main_file = QFileDialog.getOpenFileName(self, 'Choose Project Main .cxs file',current_dir)[0]
            print("newmain: " + new_main_file)
            if not new_main_file:  # If Cancel was clicked, just exit app.
                print("cancel and exit")
                sys.exit()

            elif self.check_project_mainfile(new_main_file):
                print("New ProjDir: " + new_main_file)
                self.ui.lineEdit_ProjectMainFile.setText(new_main_file)
                self.lineEdit_ProjectMainFile_editingFinished()
                break
            elif not self.check_project_mainfile(new_main_file):
                QMessageBox.warning(self, "No main .cxs file!", "You didn't choose a valid .cxs file.")

    def lineEdit_ProjectName_textChanged(self):
        # Only change the main one in each tab. The rest is done on their change signal.
        text=self.ui.lineEdit_ProjectName.text()
        if self.ui.checkBox_HtmlAppName.isChecked():
            self.ui.lineEdit_HtmlAppName.setText(text)
        if self.ui.checkBox_GlfwAppName.isChecked():
            self.ui.lineEdit_GlfwAppName.setText(text)

    def lineEdit_ProjectName_textEdited(self):
        # Uncheck checkbox, if text is edited by hand to prevent overwriting.
        if self.ui.checkBox_ProjectName.isChecked():
            self.ui.checkBox_ProjectName.setChecked(False)

    # Html Tab
    def lineEdit_HtmlAppName_textChanged(self):
        # Only change the main one in each tab. The rest is done on their change signal.
        text=self.ui.lineEdit_HtmlAppName.text()
        if self.ui.checkBox_HtmlFileName.isChecked():
            self.ui.lineEdit_HtmlFileName.setText(text)
        if self.ui.checkBox_HtmlDirName.isChecked():
            self.ui.lineEdit_HtmlDirName.setText(text)

    def lineEdit_HtmlAppName_textEdited(self):
        print("text edited")
        # Uncheck checkbox, if text is edited by hand to prevent overwriting.
        if self.ui.checkBox_HtmlAppName.isChecked():
            self.ui.checkBox_HtmlAppName.setChecked(False)

    def toolButton_HtmlFaviconFile_clicked(self):
        file = QFileDialog.getOpenFileName(self, 'Choose Favicon file (.ico)', self.ui.lineEdit_HtmlFaviconFile.text())
        print(file)
        if file[0] != "" and os.path.isfile(file[0]):
            self.ui.lineEdit_HtmlFaviconFile.setText(file[0])

    def pushButton_HtmlMakeProduct_clicked(self):
        html_target.makeProduct(self.ui)


    # Glfw Tab
    def lineEdit_GlfwAppName_textChanged(self):
        app_name = self.ui.lineEdit_GlfwAppName.text()
        app_version =  self.ui.lineEdit_GlfwAppVersion.text()
        if self.ui.checkBox_GlfwAppExeFileName.isChecked():
            self.ui.lineEdit_GlfwAppExeFileName.setText(app_name)
        if self.ui.checkBox_GlfwAppId.isChecked():
            self.ui.lineEdit_GlfwAppId.setText(app_name)
        if self.ui.checkBox_GlfwSetupPackagePath.isChecked():
            package_path = os.path.join(cfg.products_dir, app_name + '_' + app_version)
            self.ui.lineEdit_GlfwSetupPackagePath.setText(package_path)

    def lineEdit_GlfwAppName_textEdited(self):
        if self.ui.checkBox_GlfwAppName.isChecked():
            self.ui.checkBox_GlfwAppName.setChecked(False)

    def lineEdit_GlfwAppVersion_textChanged(self):
        app_name = self.ui.lineEdit_GlfwAppName.text()
        app_version =  self.ui.lineEdit_GlfwAppVersion.text()
        if self.ui.checkBox_GlfwSetupPackagePath.isChecked():
            package_path = os.path.join(cfg.products_dir, app_name + '_' + app_version)
            self.ui.lineEdit_GlfwSetupPackagePath.setText(package_path)

    def lineEdit_GlfwAppVersion_textEdited(self):
        if self.ui.checkBox_GlfwAppVersion.isChecked():
            self.ui.checkBox_GlfwAppVersion.setChecked(False)

    def lineEdit_GlfwAppExeFileName_textEdited(self):
        if self.ui.checkBox_GlfwAppExeFileName.isChecked():
            self.ui.checkBox_GlfwAppExeFileName.setChecked(False)

    def lineEdit_GlfwAppId_textEdited(self):
        if self.ui.checkBox_GlfwAppId.isChecked():
            self.ui.checkBox_GlfwAppId.setChecked(False)

    def pushButton_GlfwMakeProduct_clicked(self):
        glfw_target.makeProduct(self.ui)


def main():
    cfg.CXDeployExeDir = os.getcwd()
    cfg.CXImportsDir=os.path.join(cfg.bundle_dir, "myImports")
    print(cfg.bundle_dir)
    cfg.args = cli_functions.parse_commandline_arguments()
    app = QApplication(sys.argv)
    window = MyApp()
    window.show()
    return_value = app.exec_() #app execution starts and contains the event loop.
    sys.exit(return_value)


if __name__ == '__main__':
    main()
