# Functions for ftp server communication.

from PyQt5.QtWidgets import QMessageBox
import os.path, os
from ftplib import FTP_TLS, error_perm, all_errors



def upload_DirTree(ui,host, user, password, localDirectory, base_ftp_directory):
    try:
        ftp = FTP_TLS(host)
    except all_errors as e:
        print("ftp-Host error: {}".format(e))
        return
    try:
        ftp.login(user,password)
    except all_errors as e:
        print("ftp-Login error: {}".format(e))
        return

    with ftp:
        ftp.cwd(base_ftp_directory)
        print(ftp.nlst())
        ftp_project_dir=(os.path.split(localDirectory)[1])
        try:
            ftp.cwd(ftp_project_dir)
            print("old project dir found!")
            reply = QMessageBox.question(None, 'Warning',
                                            "Project directory {} already exists. Replace it?".format(ftp_project_dir), QMessageBox.Yes |
                                            QMessageBox.Cancel, QMessageBox.Yes)
            if reply == QMessageBox.Yes:
                ftp.cwd(base_ftp_directory)
                print("change to baseftpDir")
                removeDirTree(ftp, ftp_project_dir)
                print("old project dir removed!")
            else:
                return

        except all_errors as e:
            print("No old project dir {0} found : {1}".format(ftp_project_dir,e))

        ftp.mkd(ftp_project_dir)
        ftp.cwd(ftp_project_dir)

        placeFiles(ftp, localDirectory)
        ftp.quit()
        print("ftp finished")


def placeFiles(ftp, path):
    for name in os.listdir(path):
        local_path = os.path.join(path, name)
        if os.path.isfile(local_path):
            print("STOR", name, local_path)
            ftp.storbinary('STOR ' + name, open(local_path,'rb'))
        elif os.path.isdir(local_path):
            print("MKD", name)
            try:
                ftp.mkd(name)

            # ignore "directory already exists"
            except error_perm as e:
                if not e.args[0].startswith('550'):
                    raise

            print("CWD", name)
            ftp.cwd(name)
            placeFiles(ftp, local_path)
            print("CWD", "..")
            ftp.cwd("..")


def removeDirTree(ftp, path):
    # Recursively delete a directory tree on a remote server.
    wd = ftp.pwd()

    try:
        names = ftp.nlst(path)
    except all_errors as e:
        # some FTP servers complain when you try and list non-existent paths
        print('removeDirTree: Could not list {0}: {1}'.format(path, e))
        return

    for name in names:
        if os.path.split(name)[1] in ('.', '..'): continue

        print('removeDirTree: Checking {0}'.format(name))

        try:
            ftp.cwd(name)  # if we can cwd to it, it's a folder
            ftp.cwd(wd)  # don't try a nuke a folder we're in
            removeDirTree(ftp, name)
        except all_errors:
            ftp.delete(name)

    try:
        ftp.rmd(path)
    except all_errors as e:
        print('removeDirTree: Could not remove directory {0}: {1}'.format(path, e))


def upload_file(host, user, password, local_file, ftp_directory):
    try:
        ftp = FTP_TLS(host)
    except all_errors as e:
        print("ftp-Host error: {}".format(e))
        return
    try:
        ftp.login(user,password)
    except all_errors as e:
        print("ftp-Login error: {}".format(e))
        return

    with ftp:
        ftp.cwd(ftp_directory)

        if os.path.isfile(local_file):
            name=os.path.split(local_file)[1]
            print("STOR", name, local_file)
            msg=ftp.storbinary('STOR ' + name, open(local_file, 'rb'))
            print(msg)


