# All Functions to make the Glfw Desktop product

import os
import platform
import shutil

import cfg
import ftp_functions
from general_functions import *


def makeProduct(ui):

    relbuildDir = ui.comboBox_RelBuildDir.currentText()
    relGlfwBuildDir=ui.comboBox_GlfwRelDir.currentText()
    glfwBuildDir = os.path.join(cfg.project_dir, relbuildDir, relGlfwBuildDir)

    if not os.path.isdir(glfwBuildDir):
        print("glfwBuildDir not found!")
        return

    # Check for .cerberusstate file and create it, if missing. (needed for script)
    cerberusstate_file=os.path.join(glfwBuildDir,".cerberusstate")
    if not os.path.isfile(cerberusstate_file):
        with open(cerberusstate_file,"w"):
            pass

    os.chdir(cfg.products_dir) # Important, because cwd can not be removed.

    # Gather all information for InnoSetup Script
    parameters = {}
    parameters["CXWinBuildFolder"]=glfwBuildDir
    parameters["CXAppName"] = ui.lineEdit_GlfwAppName.text()
    parameters["CXAppVersion"] = ui.lineEdit_GlfwAppName.text()
    parameters["CXAppExeName"] = ui.lineEdit_GlfwAppExeFileName.text()
    parameters["CXAppId"] = ui.lineEdit_GlfwAppId.text()
    parameters["CXAppPublisher"] = ui.lineEdit_GlfwAppPublisher.text()
    parameters["CXAppPublisherURL"] = ui.lineEdit_GlfwAppPublisherUrl.text()
    package_file = ui.lineEdit_GlfwSetupPackagePath.text()
    parameters["CXSetupFileDir"] = os.path.split(package_file)[0] #+ "\\"
    parameters["CXSetupFileName"] = os.path.split(package_file)[1]

    command_list = ["iscc"]
    for p in parameters:
        print('\"/D' + p + '=' + parameters[p] + '\"')
        #command_list.append('\"/D' + p + '=' + parameters[p] + '\"')
        command_list.append('/D' + p + '=' + parameters[p])

    script = os.path.join(cfg.bundle_dir,"myImports", "CerberusX.iss")
    print(script)
    command_list.append(script)
    print(command_list)
    execute_commandline_process(command_list)
    open_folder(cfg.products_dir)

    # Upload html folder to server via ftps
    if ui.checkBox_GlfwFtpAutoUpload.isChecked():
        host = ui.lineEdit_FtpHost.text()
        user = ui.lineEdit_FtpUser.text()
        password = ui.lineEdit_FtpPassword.text()
        ftpDirectory = ui.lineEdit_GlfwFtpRelDir.text()
        local_file = package_file + ".exe"
        print(host,user,password,local_file, ftpDirectory)
        ftp_functions.upload_file(host,user,password,local_file,ftpDirectory)
